@@ -2,11 +2,11 @@ import dataclasses
 
 import jax
 from jax import numpy as jnp
-from overrides import overrides
+from overrides import final, overrides
 
 from . import _base, types
 from ._so3 import SO3
-from ._utils import get_epsilon, register_lie_group
+from .utils import get_epsilon, register_lie_group
 
 
 def _skew(omega: types.Vector) -> types.Matrix:
@@ -29,7 +29,7 @@ def _skew(omega: types.Vector) -> types.Matrix:
     space_dim=3,
 )
 @dataclasses.dataclass(frozen=True)
-class SE3(_base.MatrixLieGroup):
+class SE3(_base.SEBase):
     """Special Euclidean group for proper rigid transforms in 3D."""
 
     # SE3-specific
@@ -37,13 +37,18 @@ class SE3(_base.MatrixLieGroup):
     wxyz_xyz: types.Vector
     """Internal parameters. wxyz quaternion followed by xyz translation."""
 
+    @final
     @overrides
     def __repr__(self) -> str:
         quat = jnp.round(self.wxyz_xyz[..., :4], 5)
         trans = jnp.round(self.wxyz_xyz[..., 4:], 5)
         return f"{self.__class__.__name__}(wxyz={quat}, xyz={trans})"
 
+    # SE-specific
+
     @staticmethod
+    @final
+    @overrides
     def from_rotation_and_translation(
         rotation: SO3,
         translation: types.Vector,
@@ -51,22 +56,26 @@ class SE3(_base.MatrixLieGroup):
         assert translation.shape == (3,)
         return SE3(wxyz_xyz=jnp.concatenate([rotation.wxyz, translation]))
 
-    @property
+    @final
+    @overrides
     def rotation(self) -> SO3:
         return SO3(wxyz=self.wxyz_xyz[..., :4])
 
-    @property
+    @final
+    @overrides
     def translation(self) -> types.Vector:
         return self.wxyz_xyz[..., 4:]
 
     # Factory
 
     @staticmethod
+    @final
     @overrides
     def identity() -> "SE3":
         return SE3(wxyz_xyz=jnp.array([1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]))
 
     @staticmethod
+    @final
     @overrides
     def from_matrix(matrix: types.Matrix) -> "SE3":
         assert matrix.shape == (4, 4)
@@ -78,36 +87,26 @@ class SE3(_base.MatrixLieGroup):
 
     # Accessors
 
+    @final
     @overrides
     def as_matrix(self) -> types.Matrix:
         return (
             jnp.eye(4)
             .at[:3, :3]
-            .set(self.rotation.as_matrix())
+            .set(self.rotation().as_matrix())
             .at[:3, 3]
-            .set(self.translation)
+            .set(self.translation())
         )
 
-    @property  # type: ignore
+    @final
     @overrides
     def parameters(self) -> types.Vector:
         return self.wxyz_xyz
 
     # Operations
 
-    @overrides
-    def apply(self: "SE3", target: types.Vector) -> types.Vector:
-        assert target.shape == (3,)
-        return self.rotation @ target + self.translation
-
-    @overrides
-    def multiply(self: "SE3", other: "SE3") -> "SE3":
-        return SE3.from_rotation_and_translation(
-            rotation=self.rotation @ other.rotation,
-            translation=(self.rotation @ other.translation) + self.translation,
-        )
-
     @staticmethod
+    @final
     @overrides
     def exp(tangent: types.TangentVector) -> "SE3":
         # Reference:
@@ -139,11 +138,12 @@ class SE3(_base.MatrixLieGroup):
             translation=V @ tangent[:3],
         )
 
+    @final
     @overrides
     def log(self: "SE3") -> types.TangentVector:
         # Reference:
         # > https://github.com/strasdat/Sophus/blob/a0fe89a323e20c42d3cecb590937eb7a06b8343a/sophus/se3.hpp#L223
-        omega = self.rotation.log()
+        omega = self.rotation().log()
         theta_squared = omega @ omega
         skew_omega = _skew(omega)
         theta = jnp.sqrt(theta_squared)
@@ -160,34 +160,21 @@ class SE3(_base.MatrixLieGroup):
                 * (skew_omega @ skew_omega)
             ),
         )
-        return jnp.concatenate([V_inv @ self.translation, omega])
+        return jnp.concatenate([V_inv @ self.translation(), omega])
 
+    @final
     @overrides
     def adjoint(self: "SE3") -> types.Matrix:
-        R = self.rotation.as_matrix()
+        R = self.rotation().as_matrix()
         return jnp.block(
             [
-                [R, _skew(self.translation) @ R],
+                [R, _skew(self.translation()) @ R],
                 [jnp.zeros((3, 3)), R],
             ]
         )
 
-    @overrides
-    def inverse(self: "SE3") -> "SE3":
-        R_inv = self.rotation.inverse()
-        return SE3.from_rotation_and_translation(
-            rotation=R_inv,
-            translation=-(R_inv @ self.translation),
-        )
-
-    @overrides
-    def normalize(self: "SE3") -> "SE3":
-        return SE3.from_rotation_and_translation(
-            rotation=self.rotation.normalize(),
-            translation=self.translation,
-        )
-
     @staticmethod
+    @final
     @overrides
     def sample_uniform(key: jnp.ndarray) -> "SE3":
         key0, key1 = jax.random.split(key)
